package fr.sanchezm.attestationsCovid19.utilities

import android.content.res.AssetManager
import android.util.Log
import com.tom_roush.pdfbox.pdmodel.PDDocument
import com.tom_roush.pdfbox.pdmodel.interactive.form.PDAcroForm
import com.tom_roush.pdfbox.pdmodel.interactive.form.PDCheckbox
import com.tom_roush.pdfbox.pdmodel.interactive.form.PDTextField
import fr.sanchezm.attestationsCovid19.data.db.entity.Attestation
import java.io.File
import java.io.IOException
import java.util.*
import kotlin.collections.ArrayList


class PdfUtils private constructor(
    private val assetManager: AssetManager,
    private val path: String
) {

    private val folderPath = "attestations"
    private val checkboxListNames = ArrayList<String>()

    fun fillForm(attestation: Attestation) {
        checkFolder()
        try {
            // Load the document and get the AcroForm
            val document: PDDocument = PDDocument.load(assetManager.open("attestation.pdf"))
            val docCatalog = document.documentCatalog
            val acroForm = docCatalog.acroForm

            // Adding data
            setFieldsData(acroForm, attestation)
            setCheckboxFields(acroForm, attestation)
            attestation.createAt = Date().time

            val savePath = "$path/$folderPath/${attestation.createAt}.pdf"

            Log.v("PdfUtils", "Save File to $savePath")
            document.save(savePath)
            document.close()
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    private fun setFieldsData(acroForm: PDAcroForm, attestation: Attestation) {
        val profile = attestation.profile

        // Init Fields
        val namesField = acroForm.getField("Nom et prénom") as PDTextField
        val birthdayField = acroForm.getField("Date de naissance") as PDTextField
        val birthPlaceField = acroForm.getField("Lieu de naissance") as PDTextField
        val addressField = acroForm.getField("Adresse actuelle") as PDTextField
        val cityField = acroForm.getField("Ville") as PDTextField
        val dateField = acroForm.getField("Date") as PDTextField
        val hourField = acroForm.getField("Heure") as PDTextField
        val minutesField = acroForm.getField("Minute") as PDTextField
//            val signatureField = acroForm.getField("Signature") as PDTextField

        namesField.value = "${profile.firstName} ${profile.lastName}"
        birthdayField.value = profile.birthPlace
        birthPlaceField.value = profile.birthPlace
        addressField.value = "${profile.address} ${profile.postalCode} ${profile.city}"
        cityField.value = profile.city
        dateField.value = attestation.exitDate
        hourField.value = attestation.exitHour.split(":")[0]
        minutesField.value = attestation.exitHour.split(":")[1]

    }

    private fun setCheckboxFields(acroForm: PDAcroForm, attestation: Attestation) {
        attestation.reasons.forEach {
            (acroForm.getField(checkboxListNames[it - 1]) as PDCheckbox).check()
        }
    }

    private fun checkFolder() {
        val file = "$path/$folderPath"

        if (!File(file).exists()) {
            File(file).mkdir()
        }
    }

    companion object {
        @Volatile
        private var instance: PdfUtils? = null
        private val LOCK = Any()

        fun getInstance() = instance

        fun getInstance(assetManager: AssetManager, path: String) = instance ?: synchronized(LOCK) {
            instance ?: PdfUtils(assetManager, path).also { instance = it; it.initList() }
        }
    }

    private fun initList() {
        checkboxListNames.add("Déplacements entre domicile et travail")
        checkboxListNames.add("Déplacements achats nécéssaires")
        checkboxListNames.add("Consultations et soins")
        checkboxListNames.add("Déplacements pour motif familial")
        checkboxListNames.add("Déplacements brefs (activité physique et animaux)")
        checkboxListNames.add("Convcation judiciaire ou administrative")
        checkboxListNames.add("Mission d'intérêt général")
    }
}